@@ -8,22 +8,28 @@ not. They can also have outcomes, such as whether they passed or failed.
 Build objects can be associated with Results and Artifacts.
 """
 
+from __future__ import annotations
+
 import time
 import logging
 import warnings
 import datetime
 
 from time import sleep
+from typing import Iterator, List, Dict, Any
+
 import pytz
 from jenkinsapi import config
 from jenkinsapi.artifact import Artifact
+
+# from jenkinsapi.job import Job
 from jenkinsapi.result_set import ResultSet
 from jenkinsapi.jenkinsbase import JenkinsBase
 from jenkinsapi.constants import STATUS_SUCCESS
 from jenkinsapi.custom_exceptions import NoResults
 from jenkinsapi.custom_exceptions import JenkinsAPIException
 
-from six.moves.urllib.parse import quote
+from urllib.parse import quote
 from requests import HTTPError
 
 
@@ -31,7 +37,6 @@ log = logging.getLogger(__name__)
 
 
 class Build(JenkinsBase):
-
     """
     Represents a Jenkins build, executed in context of a job.
     """
@@ -41,7 +46,9 @@ class Build(JenkinsBase):
         "%s has status %s, and does not have " "any test results"
     )
 
-    def __init__(self, url, buildno, job, depth=1):
+    def __init__(
+        self, url: str, buildno: int, job: "Job", depth: int = 1
+    ) -> None:
         """
         depth=1 is for backward compatibility consideration
 
@@ -50,9 +57,8 @@ class Build(JenkinsBase):
         information, see
         https://www.jenkins.io/doc/book/using/remote-access-api/#RemoteaccessAPI-Depthcontrol
         """
-        assert isinstance(buildno, int)
-        self.buildno = buildno
-        self.job = job
+        self.buildno: int = buildno
+        self.job: "Job" = job
         self.depth = depth
         JenkinsBase.__init__(self, url)
 
@@ -63,39 +69,37 @@ class Build(JenkinsBase):
         url = self.python_api_url(self.baseurl)
         return self.get_data(url, params={"depth": self.depth}, tree=tree)
 
-    def __str__(self):
+    def __str__(self) -> str:
         return self._data["fullDisplayName"]
 
     @property
     def name(self):
         return str(self)
 
-    def get_description(self):
+    def get_description(self) -> str:
         return self._data["description"]
 
-    def get_number(self):
+    def get_number(self) -> int:
         return self._data["number"]
 
-    def get_status(self):
+    def get_status(self) -> str:
         return self._data["result"]
 
-    def get_slave(self):
+    def get_slave(self) -> str:
         return self._data["builtOn"]
 
-    def get_revision(self):
-        return getattr(self, "_get_%s_rev" % self._get_vcs(), lambda: None)()
+    def get_revision(self) -> str:
+        return getattr(self, f"_get_{self._get_vcs()}_rev", lambda: "")()
 
-    def get_revision_branch(self):
+    def get_revision_branch(self) -> str:
         return getattr(
-            self, "_get_%s_rev_branch" % self._get_vcs(), lambda: None
+            self, f"_get_{self._get_vcs()}_rev_branch", lambda: ""
         )()
 
-    def get_repo_url(self):
-        return getattr(
-            self, "_get_%s_repo_url" % self._get_vcs(), lambda: None
-        )()
+    def get_repo_url(self) -> str:
+        return getattr(self, f"_get_{self._get_vcs()}_repo_url", lambda: "")()
 
-    def get_params(self):
+    def get_params(self) -> dict[str, str]:
         """
         Return a dictionary of params names and their values, or an
         empty dictionary if no parameters are returned.
@@ -149,7 +153,7 @@ class Build(JenkinsBase):
                 return self._data["changeSets"]["items"]
         return []
 
-    def _get_vcs(self):
+    def _get_vcs(self) -> str:
         """
         Returns a string VCS.
         By default, 'git' will be used.
@@ -161,17 +165,7 @@ class Build(JenkinsBase):
             vcs = self._data["changeSets"]["kind"] or "git"
         return vcs
 
-    def _get_svn_rev(self):
-        warnings.warn(
-            "This untested function may soon be removed from Jenkinsapi "
-            "(get_svn_rev)."
-        )
-        maxRevision = 0
-        for repoPathSet in self._data["changeSet"]["revisions"]:
-            maxRevision = max(repoPathSet["revision"], maxRevision)
-        return maxRevision
-
-    def _get_git_rev(self):
+    def _get_git_rev(self) -> str | None:
         # Sometimes we have None as part of actions. Filter those actions
         # which have lastBuiltRevision in them
         _actions = [
@@ -183,21 +177,7 @@ class Build(JenkinsBase):
 
         return None
 
-    def _get_hg_rev(self):
-        warnings.warn(
-            "This untested function may soon be removed from Jenkinsapi "
-            "(_get_hg_rev)."
-        )
-        return [
-            x["mercurialNodeName"]
-            for x in self._data["actions"]
-            if "mercurialNodeName" in x
-        ][0]
-
-    def _get_svn_rev_branch(self):
-        raise NotImplementedError("_get_svn_rev_branch is not yet implemented")
-
-    def _get_git_rev_branch(self):
+    def _get_git_rev_branch(self) -> str:
         # Sometimes we have None as part of actions. Filter those actions
         # which have lastBuiltRevision in them
         _actions = [
@@ -206,10 +186,7 @@ class Build(JenkinsBase):
 
         return _actions[0]["lastBuiltRevision"]["branch"]
 
-    def _get_hg_rev_branch(self):
-        raise NotImplementedError("_get_hg_rev_branch is not yet implemented")
-
-    def _get_git_repo_url(self):
+    def _get_git_repo_url(self) -> str:
         # Sometimes we have None as part of actions. Filter those actions
         # which have lastBuiltRevision in them
         _actions = [
@@ -222,19 +199,13 @@ class Build(JenkinsBase):
             result = ",".join(result)
         return result
 
-    def _get_svn_repo_url(self):
-        raise NotImplementedError("_get_svn_repo_url is not yet implemented")
-
-    def _get_hg_repo_url(self):
-        raise NotImplementedError("_get_hg_repo_url is not yet implemented")
-
-    def get_duration(self):
+    def get_duration(self) -> datetime.timedelta:
         return datetime.timedelta(milliseconds=self._data["duration"])
 
-    def get_build_url(self):
+    def get_build_url(self) -> str:
         return self._data["url"]
 
-    def get_artifacts(self):
+    def get_artifacts(self) -> Iterator[Artifact]:
         data = self.poll(tree="artifacts[relativePath,fileName]")
         for afinfo in data["artifacts"]:
             url = "%s/artifact/%s" % (
@@ -249,10 +220,10 @@ class Build(JenkinsBase):
             )
             yield af
 
-    def get_artifact_dict(self):
-        return dict((af.relative_path, af) for af in self.get_artifacts())
+    def get_artifact_dict(self) -> dict[str, Artifact]:
+        return {af.relative_path: af for af in self.get_artifacts()}
 
-    def get_upstream_job_name(self):
+    def get_upstream_job_name(self) -> str | None:
         """
         Get the upstream job name if it exist, None otherwise
         :return: String or None
@@ -262,7 +233,7 @@ class Build(JenkinsBase):
         except KeyError:
             return None
 
-    def get_upstream_job(self):
+    def get_upstream_job(self) -> Job | None:
         """
         Get the upstream job object if it exist, None otherwise
         :return: Job or None
@@ -271,7 +242,7 @@ class Build(JenkinsBase):
             return self.get_jenkins_obj().get_job(self.get_upstream_job_name())
         return None
 
-    def get_upstream_build_number(self):
+    def get_upstream_build_number(self) -> int | None:
         """
         Get the upstream build number if it exist, None otherwise
         :return: int or None
@@ -281,18 +252,18 @@ class Build(JenkinsBase):
         except KeyError:
             return None
 
-    def get_upstream_build(self):
+    def get_upstream_build(self) -> "Build" | None:
         """
         Get the upstream build if it exist, None otherwise
         :return Build or None
         """
-        upstream_job = self.get_upstream_job()
+        upstream_job: "Job" = self.get_upstream_job()
         if upstream_job:
             return upstream_job.get_build(self.get_upstream_build_number())
 
         return None
 
-    def get_master_job_name(self):
+    def get_master_job_name(self) -> str | None:
         """
         Get the master job name if it exist, None otherwise
         :return: String or None
@@ -302,59 +273,43 @@ class Build(JenkinsBase):
         except KeyError:
             return None
 
-    def get_master_job(self):
+    def get_master_job(self) -> Job | None:
         """
         Get the master job object if it exist, None otherwise
         :return: Job or None
         """
-        warnings.warn(
-            "This untested function may soon be removed from Jenkinsapi "
-            "(get_master_job)."
-        )
         if self.get_master_job_name():
             return self.get_jenkins_obj().get_job(self.get_master_job_name())
 
         return None
 
-    def get_master_build_number(self):
+    def get_master_build_number(self) -> int | None:
         """
         Get the master build number if it exist, None otherwise
         :return: int or None
         """
-        warnings.warn(
-            "This untested function may soon be removed from Jenkinsapi "
-            "(get_master_build_number)."
-        )
         try:
             return int(self.get_actions()["parameters"][1]["value"])
         except KeyError:
             return None
 
-    def get_master_build(self):
+    def get_master_build(self) -> "Build" | None:
         """
         Get the master build if it exist, None otherwise
         :return Build or None
         """
-        warnings.warn(
-            "This untested function may soon be removed from Jenkinsapi "
-            "(get_master_build)."
-        )
-        master_job = self.get_master_job()
+        master_job: Job | None = self.get_master_job()
         if master_job:
             return master_job.get_build(self.get_master_build_number())
 
         return None
 
-    def get_downstream_jobs(self):
+    def get_downstream_jobs(self) -> List[Job]:
         """
         Get the downstream jobs for this build
         :return List of jobs or None
         """
-        warnings.warn(
-            "This untested function may soon be removed from Jenkinsapi "
-            "(get_downstream_jobs)."
-        )
-        downstream_jobs = []
+        downstream_jobs: List[Job] = []
         try:
             for job_name in self.get_downstream_job_names():
                 downstream_jobs.append(
@@ -364,13 +319,13 @@ class Build(JenkinsBase):
         except (IndexError, KeyError):
             return []
 
-    def get_downstream_job_names(self):
+    def get_downstream_job_names(self) -> List[str]:
         """
         Get the downstream job names for this build
         :return List of string or None
         """
-        downstream_job_names = self.job.get_downstream_job_names()
-        downstream_names = []
+        downstream_job_names: List[str] = self.job.get_downstream_job_names()
+        downstream_names: List[str] = []
         try:
             fingerprints = self._data["fingerprint"]
             for fingerprint in fingerprints:
@@ -381,13 +336,13 @@ class Build(JenkinsBase):
         except (IndexError, KeyError):
             return []
 
-    def get_downstream_builds(self):
+    def get_downstream_builds(self) -> List["Build"]:
         """
         Get the downstream builds for this build
         :return List of Build or None
         """
-        downstream_job_names = self.get_downstream_job_names()
-        downstream_builds = []
+        downstream_job_names: List[str] = self.get_downstream_job_names()
+        downstream_builds: List[Build] = []
         try:  # pylint: disable=R1702
             fingerprints = self._data["fingerprint"]
             for fingerprint in fingerprints:
@@ -405,7 +360,7 @@ class Build(JenkinsBase):
         except (IndexError, KeyError):
             return []
 
-    def get_matrix_runs(self):
+    def get_matrix_runs(self) -> Iterator["Build"]:
         """
         For a matrix job, get the individual builds for each
         matrix configuration
@@ -413,22 +368,22 @@ class Build(JenkinsBase):
         """
         if "runs" in self._data:
             for rinfo in self._data["runs"]:
-                number = rinfo["number"]
+                number: int = rinfo["number"]
                 if number == self._data["number"]:
                     yield Build(rinfo["url"], number, self.job)
 
-    def is_running(self):
+    def is_running(self) -> bool:
         """
         Return a bool if running.
         """
         data = self.poll(tree="building")
         return data.get("building", False)
 
-    def block(self):
+    def block(self) -> None:
         while self.is_running():
             time.sleep(1)
 
-    def is_good(self):
+    def is_good(self) -> bool:
         """
         Return a bool, true if the build was good.
         If the build is still running, return False.
@@ -437,11 +392,10 @@ class Build(JenkinsBase):
             "result"
         ] == STATUS_SUCCESS
 
-    def block_until_complete(self, delay=15):
-        assert isinstance(delay, int)
-        count = 0
+    def block_until_complete(self, delay: int = 15) -> None:
+        count: int = 0
         while self.is_running():
-            total_wait = delay * count
+            total_wait: int = delay * count
             log.info(
                 msg="Waited %is for %s #%s to complete"
                 % (total_wait, self.job.name, self.name)
@@ -449,55 +403,58 @@ class Build(JenkinsBase):
             sleep(delay)
             count += 1
 
-    def get_jenkins_obj(self):
+    def get_jenkins_obj(self) -> "Jenkins":
         return self.job.get_jenkins_obj()
 
-    def get_result_url(self):
+    def get_result_url(self) -> str:
         """
         Return the URL for the object which provides the job's result summary.
         """
-        url_tpl = r"%stestReport/%s"
+        url_tpl: str = r"%stestReport/%s"
         return url_tpl % (self._data["url"], config.JENKINS_API)
 
-    def get_resultset(self):
+    def get_resultset(self) -> ResultSet:
         """
         Obtain detailed results for this build.
+
+        Raises NoResults if the build has no results.
+
+        :return: ResultSet
         """
-        result_url = self.get_result_url()
+        result_url: str = self.get_result_url()
         if self.STR_TOTALCOUNT not in self.get_actions():
             raise NoResults(
                 "%s does not have any published results" % str(self)
             )
-        buildstatus = self.get_status()
+        buildstatus: str = self.get_status()
         if not self.get_actions()[self.STR_TOTALCOUNT]:
             raise NoResults(
                 self.STR_TPL_NOTESTS_ERR % (str(self), buildstatus)
             )
-        obj_results = ResultSet(result_url, build=self)
-        return obj_results
+        return ResultSet(result_url, build=self)
 
-    def has_resultset(self):
+    def has_resultset(self) -> bool:
         """
         Return a boolean, true if a result set is available. false if not.
         """
         return self.STR_TOTALCOUNT in self.get_actions()
 
-    def get_actions(self):
-        all_actions = {}
+    def get_actions(self) -> Dict[str, Any]:
+        all_actions: Dict[str, Any] = {}
         for dct_action in self._data["actions"]:
             if dct_action is None:
                 continue
             all_actions.update(dct_action)
         return all_actions
 
-    def get_causes(self):
+    def get_causes(self) -> List[str]:
         """
         Returns a list of causes. There can be multiple causes lists and
         some of the can be empty. For instance, when a build is manually
         aborted, Jenkins could add an empty causes list to the actions
         dict. Empty ones are ignored.
         """
-        all_causes = []
+        all_causes: List[str] = []
         for dct_action in self._data["actions"]:
             if dct_action is None:
                 continue
@@ -505,7 +462,7 @@ class Build(JenkinsBase):
                 all_causes.extend(dct_action["causes"])
         return all_causes
 
-    def get_timestamp(self):
+    def get_timestamp(self) -> datetime.datetime:
         """
         Returns build timestamp in UTC
         """
@@ -515,29 +472,30 @@ class Build(JenkinsBase):
         )
         return pytz.utc.localize(naive_timestamp)
 
-    def get_console(self):
+    def get_console(self) -> str:
         """
         Return the current state of the text console.
         """
-        url = "%s/consoleText" % self.baseurl
-        content = self.job.jenkins.requester.get_url(url).content
+        url: str = "%s/consoleText" % self.baseurl
+        resp = self.job.jenkins.requester.get_url(url)
+        content: Any = resp.content
         # This check was made for Python 3.x
         # In this version content is a bytes string
         # By contract this function must return string
         if isinstance(content, str):
             return content
         elif isinstance(content, bytes):
-            return content.decode("ISO-8859-1")
+            return content.decode(resp.encoding or "ISO-8859-1")
         else:
             raise JenkinsAPIException("Unknown content type for console")
 
-    def stream_logs(self, interval=0):
+    def stream_logs(self, interval=0) -> Iterator[str]:
         """
         Return generator which streams parts of text console.
         """
-        url = "%s/logText/progressiveText" % self.baseurl
-        size = 0
-        more_data = True
+        url: str = "%s/logText/progressiveText" % self.baseurl
+        size: int = 0
+        more_data: bool = True
         while more_data:
             resp = self.job.jenkins.requester.get_url(
                 url, params={"start": size}
@@ -547,7 +505,7 @@ class Build(JenkinsBase):
                 if isinstance(content, str):
                     yield content
                 elif isinstance(content, bytes):
-                    yield content.decode("ISO-8859-1")
+                    yield content.decode(resp.encoding or "ISO-8859-1")
                 else:
                     raise JenkinsAPIException(
                         "Unknown content type for console"
@@ -556,7 +514,7 @@ class Build(JenkinsBase):
             more_data = resp.headers.get("X-More-Data")
             sleep(interval)
 
-    def get_estimated_duration(self):
+    def get_estimated_duration(self) -> int | None:
         """
         Return the estimated build duration (in seconds) or none.
         """
@@ -566,14 +524,13 @@ class Build(JenkinsBase):
         except KeyError:
             return None
 
-    def stop(self):
+    def stop(self) -> bool:
         """
         Stops the build execution if it's running
-        :return boolean True if succeded False otherwise or the build
-            is not running
+        :return: boolean True if succeeded False otherwis
         """
         if self.is_running():
-            url = "%s/stop" % self.baseurl
+            url: str = "%s/stop" % self.baseurl
             # Starting from Jenkins 2.7 stop function sometimes breaks
             # on redirect to job page. Call to stop works fine, and
             # we don't need to have job page here.
@@ -589,30 +546,30 @@ class Build(JenkinsBase):
             return True
         return False
 
-    def get_env_vars(self):
+    def get_env_vars(self) -> Dict[str, str]:
         """
         Return the environment variables.
 
         This method is using the Environment Injector plugin:
         https://wiki.jenkins-ci.org/display/JENKINS/EnvInject+Plugin
         """
-        url = self.python_api_url("%s/injectedEnvVars" % self.baseurl)
+        url: str = self.python_api_url("%s/injectedEnvVars" % self.baseurl)
         try:
             data = self.get_data(url, params={"depth": self.depth})
         except HTTPError as ex:
             warnings.warn(
-                "Make sure the Environment Injector plugin " "is installed."
+                "Make sure the Environment Injector plugin is installed."
             )
             raise ex
         return data["envMap"]
 
-    def toggle_keep(self):
+    def toggle_keep(self) -> None:
         """
         Toggle "keep this build forever" on and off
         """
-        url = "%s/toggleLogKeep" % self.baseurl
+        url: str = "%s/toggleLogKeep" % self.baseurl
         self.get_jenkins_obj().requester.post_and_confirm_status(url, data={})
         self._data = self._poll()
 
-    def is_kept_forever(self):
+    def is_kept_forever(self) -> bool:
         return self._data["keepLog"]
