@@ -1,11 +1,12 @@
 """ jenkinsapi.command_line.jenkinsapi_version
 """
-from jenkinsapi import __version__ as version
+
+import jenkinsapi
 import sys
 
 
 def main():
-    sys.stdout.write(version)
+    sys.stdout.write(jenkinsapi.__version__)
 
 
 if __name__ == "__main__":
