@@ -1,48 +1,45 @@
 """
 jenkinsapi plugins
 """
-from __future__ import print_function
 
+from __future__ import annotations
+
+from typing import Generator
 import logging
 import time
 import re
-
-try:
-    from StringIO import StringIO
-    from urllib import urlencode
-except ImportError:
-    # Python3
-    from io import BytesIO as StringIO
-    from urllib.parse import urlencode
+from io import BytesIO
+from urllib.parse import urlencode
 import json
+
 import requests
 from jenkinsapi.plugin import Plugin
 from jenkinsapi.jenkinsbase import JenkinsBase
 from jenkinsapi.custom_exceptions import UnknownPlugin
 from jenkinsapi.custom_exceptions import JenkinsAPIException
 from jenkinsapi.utils.jsonp_to_json import jsonp_to_json
-from jenkinsapi.utils.manifest import read_manifest
+from jenkinsapi.utils.manifest import Manifest, read_manifest
 
 
-log = logging.getLogger(__name__)
+log: logging.Logger = logging.getLogger(__name__)
 
 
 class Plugins(JenkinsBase):
-
     """
     Plugins class for jenkinsapi
     """
 
-    def __init__(self, url, jenkins_obj):
-        self.jenkins_obj = jenkins_obj
+    def __init__(self, url: str, jenkins_obj: "Jenkins") -> None:
+        self.jenkins_obj: "Jenkins" = jenkins_obj
         JenkinsBase.__init__(self, url)
-        # print('DEBUG: Plugins._data=', self._data)
 
-    def get_jenkins_obj(self):
+    def get_jenkins_obj(self) -> "Jenkins":
         return self.jenkins_obj
 
-    def check_updates_server(self):
-        url = "%s/pluginManager/checkUpdatesServer" % self.jenkins_obj.baseurl
+    def check_updates_server(self) -> None:
+        url: str = (
+            f"{self.jenkins_obj.baseurl}/pluginManager/checkUpdatesServer"
+        )
         self.jenkins_obj.requester.post_and_confirm_status(
             url, params={}, data={}
         )
@@ -56,35 +53,35 @@ class Plugins(JenkinsBase):
     def _poll(self, tree=None):
         return self.get_data(self.baseurl, tree=tree)
 
-    def keys(self):
+    def keys(self) -> list[str]:
         return self.get_plugins_dict().keys()
 
     __iter__ = keys
 
-    def iteritems(self):
+    def iteritems(self) -> Generator[str, "Plugin"]:
         return self._get_plugins()
 
-    def values(self):
+    def values(self) -> list["Plugin"]:
         return [a[1] for a in self.iteritems()]
 
-    def _get_plugins(self):
+    def _get_plugins(self) -> Generator[str, "Plugin"]:
         if "plugins" in self._data:
             for p_dict in self._data["plugins"]:
                 yield p_dict["shortName"], Plugin(p_dict)
 
-    def get_plugins_dict(self):
+    def get_plugins_dict(self) -> dict[str, "Plugin"]:
         return dict(self._get_plugins())
 
-    def __len__(self):
+    def __len__(self) -> int:
         return len(self.get_plugins_dict().keys())
 
-    def __getitem__(self, plugin_name):
+    def __getitem__(self, plugin_name: str) -> Plugin:
         try:
             return self.get_plugins_dict()[plugin_name]
         except KeyError:
             raise UnknownPlugin(plugin_name)
 
-    def __setitem__(self, shortName, plugin):
+    def __setitem__(self, shortName, plugin: "Plugin") -> None:
         """
         Installs plugin in Jenkins.
 
@@ -103,13 +100,13 @@ class Plugins(JenkinsBase):
             self._install_specific_version(plugin)
         self._wait_until_plugin_installed(plugin)
 
-    def _install_plugin_from_updatecenter(self, plugin):
+    def _install_plugin_from_updatecenter(self, plugin: "Plugin") -> None:
         """
         Latest versions of plugins can be installed from the update
         center (and don't need a restart.)
         """
-        xml_str = plugin.get_attributes()
-        url = (
+        xml_str: str = plugin.get_attributes()
+        url: str = (
             "%s/pluginManager/installNecessaryPlugins"
             % self.jenkins_obj.baseurl
         )
@@ -122,7 +119,7 @@ class Plugins(JenkinsBase):
         """
         Jenkins 2.x specific
         """
-        url = "%s/updateCenter/installStatus" % self.jenkins_obj.baseurl
+        url: str = "%s/updateCenter/installStatus" % self.jenkins_obj.baseurl
         status = self.jenkins_obj.requester.get_url(url)
         if status.status_code == 404:
             raise JenkinsAPIException(
@@ -141,14 +138,14 @@ class Plugins(JenkinsBase):
             return True  # Jenkins 1.X has no update_center
         return any([job for job in jobs if job["requiresRestart"] == "true"])
 
-    def _install_specific_version(self, plugin):
+    def _install_specific_version(self, plugin: "Plugin") -> None:
         """
         Plugins that are not the latest version have to be uploaded.
         """
-        download_link = plugin.get_download_link(
+        download_link: str = plugin.get_download_link(
             update_center_dict=self.update_center_dict
         )
-        downloaded_plugin = self._download_plugin(download_link)
+        downloaded_plugin: BytesIO = self._download_plugin(download_link)
         plugin_dependencies = self._get_plugin_dependencies(downloaded_plugin)
         log.debug("Installing dependencies for plugin '%s'", plugin.shortName)
         self.jenkins_obj.install_plugins(plugin_dependencies)
@@ -162,12 +159,14 @@ class Plugins(JenkinsBase):
             params={},
         )
 
-    def _get_plugin_dependencies(self, downloaded_plugin):
+    def _get_plugin_dependencies(
+        self, downloaded_plugin: BytesIO
+    ) -> list["Plugin"]:
         """
         Returns a list of all dependencies for a downloaded plugin
         """
         plugin_dependencies = []
-        manifest = read_manifest(downloaded_plugin)
+        manifest: Manifest = read_manifest(downloaded_plugin)
         manifest_dependencies = manifest.main_section.get(
             "Plugin-Dependencies"
         )
@@ -186,11 +185,11 @@ class Plugins(JenkinsBase):
         return plugin_dependencies
 
     def _download_plugin(self, download_link):
-        downloaded_plugin = StringIO()
+        downloaded_plugin = BytesIO()
         downloaded_plugin.write(requests.get(download_link).content)
         return downloaded_plugin
 
-    def _plugin_has_finished_installation(self, plugin):
+    def _plugin_has_finished_installation(self, plugin) -> bool:
         """
         Return True if installation is marked as 'Success' or
         'SuccessButRequiresRestart' in Jenkins' update_center,
@@ -210,7 +209,7 @@ class Plugins(JenkinsBase):
         except JenkinsAPIException:
             return False  # lack of update_center in Jenkins 1.X
 
-    def plugin_version_is_being_installed(self, plugin):
+    def plugin_version_is_being_installed(self, plugin) -> bool:
         """
         Return true if plugin is currently being installed.
         """
@@ -227,7 +226,7 @@ class Plugins(JenkinsBase):
             ]
         )
 
-    def plugin_version_already_installed(self, plugin):
+    def plugin_version_already_installed(self, plugin) -> bool:
         """
         Check if plugin version is already installed
         """
